// surfacechartrotationDlg.cpp : implementation file
//

#include "stdafx.h"
#include "surfacechartrotation.h"
#include "surfacechartrotationDlg.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/////////////////////////////////////////////////////////////////////////////
// CSurfacechartrotationDlg dialog

//
// Constructor
//
CSurfacechartrotationDlg::CSurfacechartrotationDlg(CWnd* pParent /*=NULL*/)
    : CDialog(CSurfacechartrotationDlg::IDD, pParent)
{
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	// 3D view angles
	m_elevationAngle = 30;
	m_rotationAngle = 45;

	// Keep track of mouse drag
	m_isDragging = false;
	m_lastMouseX = -1;
	m_lastMouseY = -1;
}

//
// Destructor
//
CSurfacechartrotationDlg::~CSurfacechartrotationDlg()
{
    delete m_ChartViewer.getChart();
}

void CSurfacechartrotationDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_ChartViewer, m_ChartViewer);
	DDX_Control(pDX, IDC_CHECK1, m_DrawFrameOnRotate);
}

BEGIN_MESSAGE_MAP(CSurfacechartrotationDlg, CDialog)
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    ON_WM_TIMER()
    ON_CONTROL(CVN_ViewPortChanged, IDC_ChartViewer, OnViewPortChanged)
	ON_CONTROL(CVN_MouseMoveChart, IDC_ChartViewer, OnMouseMoveChart)
	ON_CONTROL(BN_CLICKED, IDC_ChartViewer, OnMouseUpChart)
    ON_WM_HSCROLL()
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CSurfacechartrotationDlg message handlers

//
// Initialization
//
BOOL CSurfacechartrotationDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Set the icon for this dialog.  The framework does this automatically
    //  when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);         // Set big icon
    SetIcon(m_hIcon, FALSE);        // Set small icon

	// Set the font of the checkbox
	m_ButtonFont.CreatePointFont(100, _T("Arial"));
	m_DrawFrameOnRotate.SetFont(&m_ButtonFont);

	// Unlike most other GUI frameworks, MFC does not enable mouse capture by default.
	// We set the mouse usage to custom usage (-1) so that the CChartViewer will capture
	// the mouse on mouse down. This allows the mouse move event handler to determine if
	// the drag is initiated by the m_ChartViewer. Also, the click event will occur on 
	// mouse up, so it can be used to detect mouse up.
	m_ChartViewer.setMouseUsage(-1);
									
	// Update the viewport to display the chart
	m_ChartViewer.updateViewPort(true, false);

	return TRUE;
}

// *** code automatically generated by VC++ MFC AppWizard ***
// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CSurfacechartrotationDlg::OnPaint()
{
    if (IsIconic())
    {
        CPaintDC dc(this); // device context for painting

        SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

        // Center icon in client rectangle
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;

        // Draw the icon
        dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
        CDialog::OnPaint();
    }
}

// *** code automatically generated by VC++ MFC AppWizard ***
// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSurfacechartrotationDlg::OnQueryDragIcon()
{
    return static_cast<HCURSOR>(m_hIcon);
}

//
// View port changed event
//
void CSurfacechartrotationDlg::OnViewPortChanged()
{
    // Update the chart if necessary
    if (m_ChartViewer.needUpdateChart())
        drawChart(&m_ChartViewer);
}

/////////////////////////////////////////////////////////////////////////////
// CSurfacechartrotationDlg methods

//
// Draw the chart and display it in the given viewer
//
void CSurfacechartrotationDlg::drawChart(CChartViewer *viewer)
{
	// The x and y coordinates of the grid
	double dataX[] = { -10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
	const int dataXLength = (int)(sizeof(dataX) / sizeof(*dataX));

	double dataY[] = { -10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
	const int dataYLength = (int)(sizeof(dataY) / sizeof(*dataY));

	// The values at the grid points. In this example, we will compute the values using the
	// formula z = x * sin(y) + y * sin(x).
	double dataZ[dataXLength * dataYLength];
	for (int yIndex = 0; yIndex < dataYLength; ++yIndex)
	{
		double y = dataY[yIndex];
		for (int xIndex = 0; xIndex < dataXLength; ++xIndex)
		{
			double x = dataX[xIndex];
			dataZ[yIndex * dataXLength + xIndex] = x * sin(y) + y * sin(x);
		}
	}

	// Create a SurfaceChart object of size 720 x 600 pixels
	SurfaceChart *c = new SurfaceChart(720, 600);

	// Set the center of the plot region at (330, 290), and set width x depth x height to
	// 360 x 360 x 270 pixels
	c->setPlotRegion(330, 290, 360, 360, 270);

	// Set the data to use to plot the chart
	c->setData(DoubleArray(dataX, dataXLength), DoubleArray(dataY, dataYLength), 
		DoubleArray(dataZ, dataXLength * dataYLength));

	// Spline interpolate data to a 80 x 80 grid for a smooth surface
	c->setInterpolation(80, 80);

	// Set the view angles
	c->setViewAngle(m_elevationAngle, m_rotationAngle);

	// Check if draw frame only during rotation
	if (m_isDragging && m_DrawFrameOnRotate.GetCheck())
		c->setShadingMode(Chart::RectangularFrame);

	// Add a color axis (the legend) in which the left center is anchored at (660, 270). Set
	// the length to 200 pixels and the labels on the right side.
	c->setColorAxis(650, 270, Chart::Left, 200, Chart::Right);

	// Set the x, y and z axis titles using 10 points Arial Bold font
	c->xAxis()->setTitle("X", "arialbd.ttf", 15);
	c->yAxis()->setTitle("Y", "arialbd.ttf", 15);

	// Set axis label font
	c->xAxis()->setLabelStyle("arial.ttf", 10);
	c->yAxis()->setLabelStyle("arial.ttf", 10);
	c->zAxis()->setLabelStyle("arial.ttf", 10);
	c->colorAxis()->setLabelStyle("arial.ttf", 10);

	// Output the chart
	delete viewer->getChart();
    viewer->setChart(c);
}

//
// Rotate surface chart when mouse is moving over plotarea
//
void CSurfacechartrotationDlg::OnMouseMoveChart()
{
    int mouseX = m_ChartViewer.getChartMouseX();
	int mouseY = m_ChartViewer.getChartMouseY();

    // Drag occurs if mouse button is down and the mouse is captured by the m_ChartViewer
	if (((GetKeyState(VK_LBUTTON) & 0x100) != 0) && (GetCapture() == &m_ChartViewer))
	{
		if (m_isDragging)
		{
			// The chart is configured to rotate by 90 degrees when the mouse moves from 
			// left to right, which is the plot region width (360 pixels). Similarly, the
			// elevation changes by 90 degrees when the mouse moves from top to buttom,
			// which is the plot region height (270 pixels).
			m_rotationAngle += (m_lastMouseX - mouseX) * 90.0 / 360;
			m_elevationAngle += (mouseY - m_lastMouseY) * 90.0 / 270;
			m_ChartViewer.updateViewPort(true, false);
		}

		// Keep track of the last mouse position
		m_lastMouseX = mouseX;
		m_lastMouseY = mouseY;
		m_isDragging = true;
	}
}

//
// Stops dragging on mouse up
//
void CSurfacechartrotationDlg::OnMouseUpChart()
{
	m_isDragging = false;
	m_ChartViewer.updateViewPort(true, false);
}
